from django.contrib import admin

from custard.forms import CustomFieldModelBaseForm

from .models import Example, CustomFieldsModel, CustomValuesModel


class ExampleForm(CustomFieldModelBaseForm):

    def __init__(self, *args, **kwargs):
        super(ExampleForm, self).__init__(*args, **kwargs)
 
    def search_value_for_field(self, field, content_type, object_id):
        return CustomValuesModel.objects.filter(custom_field=field,
                                                content_type=content_type,
                                                object_id=object_id)

    def create_value_for_field(self, field, object_id, value):
        return CustomValuesModel(custom_field=field,
                                 object_id=object_id,
                                 value=value)

    def get_fields_for_content_type(self, content_type):
        return CustomFieldsModel.objects.filter(content_type=content_type)

    class Meta:
        model = Example


class ExampleAdmin(admin.ModelAdmin):
    form = ExampleForm
    search_fields = ('name',)

    def get_search_results(self, request, queryset, search_term):
        queryset, use_distinct = super(ExampleAdmin, self).get_search_results(request, queryset, search_term)
        queryset |= self.model.objects.search(search_term)
        return queryset, use_distinct

admin.site.register(Example, ExampleAdmin)


class CustomFieldsModelAdmin(admin.ModelAdmin):
    pass
admin.site.register(CustomFieldsModel, CustomFieldsModelAdmin)


class CustomValuesModelAdmin(admin.ModelAdmin):
    pass
admin.site.register(CustomValuesModel, CustomValuesModelAdmin)



# -*- coding: utf-8 -*-

from django.conf import settings


CUSTOM_TYPE_TEXT     = 'text'
CUSTOM_TYPE_INTEGER  = 'int'
CUSTOM_TYPE_FLOAT    = 'float'
CUSTOM_TYPE_TIME     = 'time'
CUSTOM_TYPE_DATE     = 'date'
CUSTOM_TYPE_DATETIME = 'datetime'
CUSTOM_TYPE_BOOLEAN  = 'boolean'


CUSTOM_CONTENT_TYPES = getattr(settings, 'CUSTOM_CONTENT_TYPES', None)


CUSTOM_FIELD_TYPES = {
    CUSTOM_TYPE_TEXT:     'django.forms.fields.CharField',
    CUSTOM_TYPE_INTEGER:  'django.forms.fields.IntegerField',
    CUSTOM_TYPE_FLOAT:    'django.forms.fields.FloatField',
    CUSTOM_TYPE_TIME:     'django.forms.fields.TimeField',
    CUSTOM_TYPE_DATE:     'django.forms.fields.DateField',
    CUSTOM_TYPE_DATETIME: 'django.forms.fields.DateTimeField',
    CUSTOM_TYPE_BOOLEAN:  'django.forms.fields.BooleanField',
}

CUSTOM_FIELD_TYPES.update(getattr(settings, 'CUSTOM_FIELD_TYPES', {}))


CUSTOM_WIDGETS_TYPES = {
    CUSTOM_TYPE_TEXT:     'django.contrib.admin.widgets.AdminTextInputWidget',
    CUSTOM_TYPE_INTEGER:  'django.contrib.admin.widgets.AdminIntegerFieldWidget',
    CUSTOM_TYPE_FLOAT:    'django.contrib.admin.widgets.AdminIntegerFieldWidget',
    CUSTOM_TYPE_TIME:     'django.contrib.admin.widgets.AdminTimeWidget',
    CUSTOM_TYPE_DATE:     'django.contrib.admin.widgets.AdminDateWidget',
    CUSTOM_TYPE_DATETIME: 'django.contrib.admin.widgets.AdminSplitDateTime',
    CUSTOM_TYPE_BOOLEAN:  'django.forms.widgets.CheckboxInput',
}

CUSTOM_WIDGETS_TYPES.update(getattr(settings, 'CUSTOM_WIDGETS_TYPES', {}))



# -*- coding: utf-8 -*-

from django.db import models
from django.db.models import Q
from django.db.models.loading import get_model
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.core.exceptions import ValidationError, NON_FIELD_ERRORS
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from .conf import (CUSTOM_TYPE_TEXT, CUSTOM_TYPE_INTEGER, CUSTOM_TYPE_FLOAT,
    CUSTOM_TYPE_TIME, CUSTOM_TYPE_DATE, CUSTOM_TYPE_DATETIME, CUSTOM_TYPE_BOOLEAN,
    CUSTOM_CONTENT_TYPES, CUSTOM_FIELD_TYPES)


#==============================================================================
class CustomContentType(object):

    def create_fields(self, base_model=models.Model):
        """
        This method will create a model which will hold field types defined
        at runtime for each ContentType.

        :param base_model:
        :return:
        """

        @python_2_unicode_compatible
        class CustomContentTypeField(base_model):
            DATATYPE_CHOICES = (
                (CUSTOM_TYPE_TEXT,     _('text')),
                (CUSTOM_TYPE_INTEGER,  _('integer')),
                (CUSTOM_TYPE_FLOAT,    _('float')),
                (CUSTOM_TYPE_TIME,     _('time')),
                (CUSTOM_TYPE_DATE,     _('date')),
                (CUSTOM_TYPE_DATETIME, _('datetime')),
                (CUSTOM_TYPE_BOOLEAN,  _('boolean')),
            )

            CONTENT_TYPES = Q(name__in=CUSTOM_CONTENT_TYPES)\
                if CUSTOM_CONTENT_TYPES is not None else Q()

            content_type = models.ForeignKey(ContentType,
                                             related_name='custom_fields',
                                             verbose_name=_('content type'),
                                             limit_choices_to=CONTENT_TYPES)
            name = models.CharField(_('name'), max_length=100, db_index=True)
            label = models.CharField(_('label'), max_length=100)
            data_type = models.CharField(_('data type'), max_length=8, choices=DATATYPE_CHOICES, db_index=True)
            help_text = models.CharField(_('help text'), max_length=200, blank=True, null=True)
            required = models.BooleanField(_('required'), default=False)
            searchable = models.BooleanField(_('searchable'), default=True)
            initial = models.CharField(_('initial'), max_length=200, blank=True, null=True)
            min_length = models.PositiveIntegerField(_('min length'), blank=True, null=True)
            max_length = models.PositiveIntegerField(_('max length'), blank=True, null=True)
            min_value = models.FloatField(_('min value'), blank=True, null=True)
            max_value = models.FloatField(_('max value'), blank=True, null=True)
        
            class Meta:
                #unique_together = ('content_type', 'name')
                verbose_name = _('custom field')
                verbose_name_plural = _('custom fields')
                abstract = True
        
            def save(self, *args, **kwargs):
                super(CustomContentTypeField, self).save(*args, **kwargs)

            def validate_unique(self, exclude=None):
                # HACK - workaround django bug https://code.djangoproject.com/ticket/17582
                #try:
                #ct = self.content_type
                #except:
                #    raise ValidationError({ NON_FIELD_ERRORS: (_('The content type has not been specified'),) })

                # field name already defined in Model class
                model = self.content_type.model_class()
                if self.name in [f.name for f in model._meta.fields]:
                    raise ValidationError({ 'name': (_('Custom field already defined for content type %(model_name)s') % {'model_name': model.__name__},) })

                # field name already defined in custom fields for content type
                qs = self.__class__._default_manager.filter(
                    content_type=self.content_type,
                    name=self.name,
                )
                if not self._state.adding and self.pk is not None:
                    qs = qs.exclude(pk=self.pk)
                if qs.exists():
                    raise ValidationError({ 'name': (_('Custom field already defined content type %(model_name)s') % {'model_name': model.__name__},) })

                # if field is required must issue a initial value
                if self.required:
                    # TODO - must create values for all instances that have not
                    #print model.objects.values_list('pk', flat=True)
                    #print self.field.filter(content_type=self.content_type)
                    #objs = self.field.filter(content_type=self.content_type) \
                    #                 .exclude(object_id__in=model.objects.values_list('pk', flat=True))
                    #for obj in objs:
                    #    print obj
                    pass

            def __str__(self):
                return "%s" % (self.name)

        return CustomContentTypeField

    def create_values(self, custom_field_model, base_model=models.Model):
        """
        This method will create a model which will hold field values for
        field types of custom_field_model.

        :param custom_field_model:
        :param base_model:
        :return:
        """

        @python_2_unicode_compatible
        class CustomContentTypeFieldValue(base_model):
            custom_field = models.ForeignKey(custom_field_model,
                                             verbose_name=_('custom field'),
                                             related_name='field')
            content_type = models.ForeignKey(ContentType, editable=False,
                                             verbose_name=_('content type'),
                                             limit_choices_to=custom_field_model.CONTENT_TYPES)
            object_id = models.PositiveIntegerField(_('object id'), db_index=True)
            content_object = generic.GenericForeignKey('content_type', 'object_id')

            value_text = models.TextField(blank=True, null=True)
            value_integer = models.IntegerField(blank=True, null=True)
            value_float = models.FloatField(blank=True, null=True)
            value_time = models.TimeField(blank=True, null=True)
            value_date = models.DateField(blank=True, null=True)
            value_datetime = models.DateTimeField(blank=True, null=True)
            value_boolean = models.NullBooleanField(blank=True)
        
            def _get_value(self):
                return getattr(self, 'value_%s' % self.custom_field.data_type)

            def _set_value(self, new_value):
                setattr(self, 'value_%s' % self.custom_field.data_type, new_value)

            value = property(_get_value, _set_value)
    
            class Meta:
                #unique_together = ('custom_field', 'content_type', 'object_id')
                verbose_name = _('custom field value')
                verbose_name_plural = _('custom field values')
                abstract = True
        
            def save(self, *args, **kwargs):
                # save content type as user shouldn't be able to change it
                self.content_type = self.custom_field.content_type
                super(CustomContentTypeFieldValue, self).save(*args, **kwargs)
        
            def validate_unique(self, exclude=None):
                qs = self.__class__._default_manager.filter(
                    custom_field=self.custom_field,
                    content_type=self.custom_field.content_type,
                    object_id=self.object_id,
                )
                if not self._state.adding and self.pk is not None:
                    qs = qs.exclude(pk=self.pk)
                if qs.exists():
                    raise ValidationError({ NON_FIELD_ERRORS: (_('A value for this custom field already exists'),) })

                # TODO - here perform custom validation of value

            def __str__(self):
                return "%s(%s): %s" % (self.custom_field.name, self.object_id, self.value)
    
        return CustomContentTypeFieldValue

    def create_manager(self, fields_model, values_model, base_manager=models.Manager):
        """
        This will create the custom Manager that will use the fields_model and values_model
        respectively.

        :param fields_model:
        :param values_model:
        :return:
        """
        fields_model = fields_model.split(".")
        values_model = values_model.split(".")

        # TODO validate fields_model / values_model strings

        class CustomManager(base_manager):
            @cached_property
            def _fields_model(self):
                return get_model(fields_model[0], fields_model[1])
        
            @cached_property
            def _values_model(self):
                return get_model(values_model[0], values_model[1])
        
            def search(self, search_data, custom_args={}):
                """
                Search inside the custom fields for this model for any match
                 of search_data and returns existing model instances

                :param search_data:
                :param custom_args:
                :return:
                """
                query = None
                content_type = ContentType.objects.get_for_model(self.model)
                custom_args = dict({ 'content_type': content_type, 'searchable': True }, **custom_args)
                custom_fields = dict((f.name, f) for f in self._fields_model.objects.filter(**custom_args))
                for key, f in custom_fields.items():
                    # TODO - search in other field types (not only text)
                    value_lookup = 'value_text'
                    value_lookup = '%s__%s' % (value_lookup, 'icontains')
                    found = self._values_model.objects.filter(**{ 'custom_field': f,
                                                                  'content_type': content_type,
                                                                  value_lookup: search_data })
                    if found.count() > 0:
                        if query is None:
                            query = Q()
                        query = query & Q(**{ str('%s__in' % self.model._meta.pk.name):
                                              [obj.object_id for obj in found] })
                if query is None:
                    return self.get_queryset().none()
                return self.get_queryset().filter(query)

        return CustomManager

    def create_mixin(self, fields_model, values_model):
        """

        :param fields_model:
        :param values_model:
        :return:
        """

        fields_model = fields_model.split(".")
        values_model = values_model.split(".")

        # TODO validate fields_model / values_model strings

        class CustomModelMixin(object):
            @cached_property
            def _fields_model(self):
                return get_model(fields_model[0], fields_model[1])

            @cached_property
            def _values_model(self):
                return get_model(values_model[0], values_model[1])

            @cached_property
            def _content_type(self):
                return ContentType.objects.get_for_model(self.__class__)

            def get_custom_fields(self):
                """ Return a list of custom fields for this model """
                return self._fields_model.objects.filter(content_type=self._content_type)

            def get_custom_field(self, field_name):
                """ Get a custom field object for this model """
                return self._fields_model.objects.get(name=field_name,
                                                      content_type=self._content_type)

            def get_custom_value(self, field_name):
                """ Get a value for a specified custom field """
                return self._values_model.objects.get(custom_field__name=field_name,
                                                      content_type=self._content_type,
                                                      object_id=self.pk).value

            def set_custom_value(self, field_name, value):
                """ Set a value for a specified custom field """
                custom_value = self._values_model.objects.get_or_create(custom_field__name=field_name,
                                                                        object_id=self.pk)[0]
                custom_value.value = value
                custom_value.save()
                return custom_value

        return CustomModelMixin

#==============================================================================
custom = CustomContentType()



from django.db import models

from custard.models import custom

# Create your models here.

CustomModelMixin = custom.create_mixin('demo.CustomFieldsModel', 'demo.CustomValuesModel')
CustomModelManager = custom.create_manager('demo.CustomFieldsModel', 'demo.CustomValuesModel')


class Example(models.Model, CustomModelMixin):
    name = models.CharField(max_length=255)

    objects = CustomModelManager()
    
    def __str__(self):
        return "%s" % self.name


class CustomFieldsModel(custom.create_fields()):
    class Meta:
        verbose_name = 'custom field'
        verbose_name_plural = 'custom fields'


class CustomValuesModel(custom.create_values(CustomFieldsModel)):
    class Meta:
        verbose_name = 'custom field value'
        verbose_name_plural = 'custom field values'



from django.db import models

from custard.models import custom

CustomMixin = custom.create_mixin('tests.CustomFieldsModel', 'tests.CustomValuesModel')
CustomManager = custom.create_manager('tests.CustomFieldsModel', 'tests.CustomValuesModel')

class SimpleModelNotRegistered(models.Model):
    name = models.CharField(max_length=255)

    class Meta:
        app_label = 'tests'

    def __str__(self):
        return "%s" % self.name


class SimpleModelWithoutManager(models.Model):
    name = models.CharField(max_length=255)

    class Meta:
        app_label = 'tests'

    def __str__(self):
        return "%s" % self.name


class SimpleModelWithManager(models.Model, CustomMixin):
    name = models.CharField(max_length=255)

    objects = CustomManager()

    class Meta:
        app_label = 'tests'

    def __str__(self):
        return "%s" % self.name


class CustomFieldsModel(custom.create_fields()):
    class Meta:
        app_label = 'tests'

class CustomValuesModel(custom.create_values(CustomFieldsModel)):
    class Meta:
        app_label = 'tests'



# -*- coding: utf-8 -*-

from django import forms
from django.contrib.contenttypes.models import ContentType

from .conf import (CUSTOM_TYPE_TEXT, CUSTOM_TYPE_INTEGER, CUSTOM_TYPE_FLOAT,
    CUSTOM_TYPE_TIME, CUSTOM_TYPE_DATE, CUSTOM_TYPE_DATETIME, CUSTOM_TYPE_BOOLEAN,
    CUSTOM_CONTENT_TYPES, CUSTOM_FIELD_TYPES, CUSTOM_WIDGETS_TYPES)
from .utils import import_class


#==============================================================================
class CustomFieldModelBaseForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        # additional form variables
        self.custom_classes = None

        # construct the form
        super(CustomFieldModelBaseForm, self).__init__(*args, **kwargs)
        
        # init custom fields from model in the form
        self.init_custom_fields()

    def clean(self):
        cleaned_data = super(CustomFieldModelBaseForm, self).clean()
        return cleaned_data

    def save(self, commit=True):
        self.instance = super(CustomFieldModelBaseForm, self).save(commit)
        if self.instance:
            self.instance.save()
        if not self.instance.pk:
            raise Exception("Cannot create an object for some reason")
        self.save_custom_fields()
        return self.instance

    def init_custom_fields(self):
        """
        Populate the ``form.fields[]`` with the additional fields coming from
        the custom fields models.
        """
        content_type = self.get_content_type()
        fields = self.get_fields_for_content_type(content_type)
        for f in fields:
            name = str(f.name)
            initial = f.initial
            self.fields[name] = self.get_formfield_for_field(f)
            self.fields[name].is_custom = True
            self.fields[name].required = f.required
            self.fields[name].widget = self.get_widget_for_field(f)
            if self.instance and self.instance.pk:
                value = self.search_value_for_field(f,
                                                    content_type,
                                                    self.instance.pk)
                if len(value) > 0:
                    initial = value[0].value
            self.fields[name].initial = self.initial[name] = initial

    def save_custom_fields(self):
        """
        Perform save and validation over the custom fields
        """
        content_type = self.get_content_type()
        fields = self.get_fields_for_content_type(content_type)
        for f in fields:
            name = str(f.name)
            fv = self.search_value_for_field(f,
                                             content_type,
                                             self.instance.pk)
            if len(fv) > 0:
                value = fv[0]
                value.value = self.cleaned_data[name]
            else:
                value = self.create_value_for_field(f,
                                                    self.instance.pk,
                                                    self.cleaned_data[name])
            value.save()

    def get_model(self):
        """
        Returns the actual model this ``ModelForm`` is referring to
        """
        return self._meta.model

    def get_content_type(self):
        """
        Returns the content type instance of the model this ``ModelForm`` is
        referring to
        """
        return ContentType.objects.get_for_model(self.get_model())

    def get_formfield_for_field(self, field):
        """
        Returns the defined formfield instance built from the type of the field

        :param field: custom field instance
        :return: the formfield instance
        """
        field_attrs = {
            'label': field.label,
            'help_text': field.help_text,
            'required': field.required,
        }
        if field.data_type == CUSTOM_TYPE_TEXT:
            #widget_attrs = {}
            if field.min_length:
                field_attrs['min_length'] = field.min_length
            if field.max_length:
                field_attrs['max_length'] = field.max_length
            #    widget_attrs['maxlength'] = field.max_length
            #field_attrs['widget'] = widgets.AdminTextInputWidget(attrs=widget_attrs)
        elif field.data_type == CUSTOM_TYPE_INTEGER:
            if field.min_value: field_attrs['min_value'] = int(float(field.min_value))
            if field.max_value: field_attrs['max_value'] = int(float(field.max_value))
            #field_attrs['widget'] = spinner.IntegerSpinnerWidget(attrs=field_attrs)
        elif field.data_type == CUSTOM_TYPE_FLOAT:
            if field.min_value: field_attrs['min_value'] = float(field.min_value)
            if field.max_value: field_attrs['max_value'] = float(field.max_value)
            #field_attrs['widget'] = spinner.SpinnerWidget(attrs=field_attrs)
        elif field.data_type == CUSTOM_TYPE_TIME:
            #field_attrs['widget'] = date.TimePickerWidget()
            pass
        elif field.data_type == CUSTOM_TYPE_DATE:
            #field_attrs['widget'] = date.DatePickerWidget()
            pass
        elif field.data_type == CUSTOM_TYPE_DATETIME:
            #field_attrs['widget'] = date.DateTimePickerWidget()
            pass
        elif field.data_type == CUSTOM_TYPE_BOOLEAN:
            pass
        field_type = import_class(CUSTOM_FIELD_TYPES[field.data_type])
        return field_type(**field_attrs)

    def get_widget_for_field(self, field, attrs={}):
        """
        Returns the defined widget type instance built from the type of the field

        :param field: custom field instance
        :param attrs: attributes of widgets
        :return: the widget instance
        """
        return import_class(CUSTOM_WIDGETS_TYPES[field.data_type])(**attrs)

    def get_fields_for_content_type(self, content_type):
        """
        Returns all fields for a given content type

        Example implementation:

          return MyCustomField.objects.filter(content_type=content_type)

        :param content_type: content type to search
        :return: the custom field instances
        """
        raise NotImplementedError

    def search_value_for_field(self, field, content_type, object_id):
        """
        This function will return the CustomFieldValue instance for a given
        field of an object that has the given content_type

        Example implementation:

          return MyCustomFieldValue.objects.filter(custom_field=field,
                                                   content_type=content_type,
                                                   object_id=object_id)

        :param field: the custom field instance
        :param content_type: the content type instance
        :param object_id: the object id this value is referring to
        :return: CustomFieldValue queryset
        """
        raise NotImplementedError

    def create_value_for_field(self, field, object_id, value):
        """
        Create a value for a given field of an object

        Example implementation:

          return MyCustomFieldValue(custom_field=field,
                                    object_id=object_id,
                                    value=value)

        :param field: the custom field instance
        :param object_id: the object id this value is referring to
        :param value: the value to set
        :return: the value instance (not saved!)
        """
        raise NotImplementedError



from setuptools import setup

from custard import VERSION

setup(
    name='django-custard',
    version=VERSION,
    license='MIT',
    url='https://github.com/kunitoki/django-custard',
    author='Lucio Asnaghi (aka kunitoki)',
    author_email='kunitoki@gmail.com',
    description='Django runtime generic customizable fields for any model.',
    long_description=open('README.rst').read(),
    packages=[
        'custard',
        'custard.tests',
    ],
    package_data={
        'custard': ['templates/custard/admin/*.html'],
    },
    install_requires=[
        "Django >= 1.6.5",
    ],
    keywords=[
        'django',
        'models',
        'fields',
        'custom',
        'admin',
        'content types'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Environment :: Web Environment',
        'Topic :: Software Development',
    ]
)



import django
#from django.conf import settings
#from django.core.urlresolvers import reverse
#from django.db import models
#from django.db.models import Q
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.test.utils import override_settings

from custard.conf import CUSTOM_TYPE_TEXT
from custard.models import custom

from .models import (SimpleModelWithManager, SimpleModelWithoutManager,
    CustomFieldsModel, CustomValuesModel)


#==============================================================================
class CustomModelsTestCase(TestCase):
        
    def setUp(self):
        self.simple_with_manager_ct = ContentType.objects.get_for_model(SimpleModelWithManager)
        self.simple_without_manager_ct = ContentType.objects.get_for_model(SimpleModelWithoutManager)

        self.cf = CustomFieldsModel.objects.create(content_type=self.simple_with_manager_ct,
                                                   name='text_field',
                                                   label='Text field',
                                                   data_type=CUSTOM_TYPE_TEXT)
        self.cf.save()

        self.cf2 = CustomFieldsModel.objects.create(content_type=self.simple_with_manager_ct,
                                                    name='another_text_field',
                                                    label='Text field 2',
                                                    data_type=CUSTOM_TYPE_TEXT,
                                                    required=True,
                                                    searchable=False)
        self.cf2.save()

        self.obj = SimpleModelWithManager.objects.create(name='old test')
        self.obj.save()

    def tearDown(self):
        CustomFieldsModel.objects.all().delete()

    #@override_settings(CUSTOM_CONTENT_TYPES=['simplemodelwithmanager'])
    #def test_creation(self):
    #    reload(custard.models)
    #    from custard.models import custom
    #    print CUSTOM_CONTENT_TYPES
    #    print settings.CUSTOM_CONTENT_TYPES
    #
    #    class TestCustomFieldsModel(custom.create_fields()):
    #        class Meta:
    #            app_label = 'tests'
    #
    #    self.assertQuerysetEqual(ContentType.objects.filter(TestCustomFieldsModel.CONTENT_TYPES),
    #                             ContentType.objects.filter(Q(name__in=['simplemodelwithmanager'])))

    #def test_get_form_fields(self):
    #    with self.settings(CUSTOM_FIELD_TYPES={CUSTOM_TYPE_TEXT: 'django.forms.fields.CharField'}):
    #        self.assertIsNotNone(self.cf.get_form_field())
    #        self.assertEqual(django.forms.fields.CharField, self.cf.get_form_field().__class__)

    def test_value_creation(self):
        val = CustomValuesModel.objects.create(custom_field=self.cf,
                                               object_id=self.obj.pk)
        val.value = "qwertyuiop"
        val.save()
        self.assertEqual(val.content_type, self.simple_with_manager_ct)
        self.assertEqual(val.content_type, val.custom_field.content_type)
        self.assertEqual(val.value_text, "qwertyuiop")
        self.assertEqual(val.value, "qwertyuiop")

    def test_value_search(self):
        newobj = SimpleModelWithManager.objects.create(name='new simple')
        newobj.save()

        v1 = CustomValuesModel.objects.create(custom_field=self.cf,
                                              object_id=self.obj.pk)
        v1.value = "qwertyuiop"
        v1.save()

        v2 = CustomValuesModel.objects.create(custom_field=self.cf,
                                              object_id=newobj.pk)
        v2.value = "qwertyasdf"
        v2.save()

        qs1 = SimpleModelWithManager.objects.search("asdf")
        self.assertQuerysetEqual(qs1, [repr(newobj)])

        qs2 = SimpleModelWithManager.objects.search("qwerty")
        self.assertQuerysetEqual(qs2, [repr(self.obj), repr(newobj)], ordered=False)

    def test_value_search_not_searchable_field(self):
        v1 = CustomValuesModel.objects.create(custom_field=self.cf,
                                              object_id=self.obj.pk)
        v1.value = "12345"
        v1.save()

        v2 = CustomValuesModel.objects.create(custom_field=self.cf2,
                                              object_id=self.obj.pk)
        v2.value = "67890"
        v2.save()

        qs1 = SimpleModelWithManager.objects.search("12345")
        self.assertQuerysetEqual(qs1, [repr(self.obj)])

        qs2 = SimpleModelWithManager.objects.search("67890")
        self.assertQuerysetEqual(qs2, [])
